==> agrum-2.1.0-1-x86_64.pkg.tar.zst: usr/include/agrum/base/graphs/PDAG_inl.h <==
/****************************************************************************
 *   This file is part of the aGrUM/pyAgrum library.                        *
 *                                                                          *
 *   Copyright (c) 2005-2025 by                                             *
 *       - Pierre-Henri WUILLEMIN(_at_LIP6)                                 *
 *       - Christophe GONZALES(_at_AMU)                                     *
 *                                                                          *
 *   The aGrUM/pyAgrum library is free software; you can redistribute it    *
 *   and/or modify it under the terms of either :                           *
 *                                                                          *
 *    - the GNU Lesser General Public License as published by               *
 *      the Free Software Foundation, either version 3 of the License,      *
 *      or (at your option) any later version,                              *
 *    - the MIT license (MIT),                                              *
 *    - or both in dual license, as here.                                   *
 *                                                                          *
 *   (see https://agrum.gitlab.io/articles/dual-licenses-lgplv3mit.html)    *
 *                                                                          *
 *   This aGrUM/pyAgrum library is distributed in the hope that it will be  *
 *   useful, but WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,          *
 *   INCLUDING BUT NOT LIMITED TO THE WARRANTIES MERCHANTABILITY or FITNESS *
 *   FOR A PARTICULAR PURPOSE  AND NONINFRINGEMENT. IN NO EVENT SHALL THE   *
 *   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER *
 *   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR  *
 *   OTHER DEALINGS IN THE SOFTWARE.                                        *
 *                                                                          *
 *   See the GNU Lesser General Public License (LICENSE.LGPL) and the MIT   *
 *   licence (LICENSE.MIT) for more details.                                *
 *                                                                          *
 *   Contact  : info_at_agrum_dot_org                                       *
 *   homepage : http://agrum.gitlab.io                                      *
 *   gitlab   : https://gitlab.com/agrumery/agrum                           *
 *                                                                          *
 ****************************************************************************/


/** @file
 * @brief Inline implementation of Base classes for directed acylic graphs
 *
 * @author Pierre-Henri WUILLEMIN(_at_LIP6) & Christophe GONZALES(_at_AMU)
 *
 */

// to ease IDE parser
#include <agrum/base/core/exceptions.h>
#include <agrum/base/graphs/PDAG.h>

namespace gum {

  INLINE PDAG& PDAG::operator=(const PDAG& g) {
    // avoid self assignment
    if (this != &g) { MixedGraph::operator=(g); }

    return *this;
  }

  INLINE void PDAG::addArc(const NodeId tail, const NodeId head) {
    if (head == tail) {
      GUM_ERROR(InvalidDirectedCycle, "Add a mono-cycle in a PDAG for node " << head)
    }
    if (this->hasDirectedPath(head, tail)) {
      GUM_ERROR(InvalidDirectedCycle,
                "Add a directed cycle in a PDAG : between " << head << " and " << tail)
    }

    // checking whether tail and head do belong to the graph is performed
    // within class DiGraph
    MixedGraph::addArc(tail, head);
  }

  INLINE void PDAG::addEdge(const NodeId tail, const NodeId head) {
    if (head == tail) { GUM_ERROR(InvalidDirectedCycle, "Add a mono-cycle in a PDAG !") }

    // checking whether tail and head do belong to the graph is performed
    // within class DiGraph
    MixedGraph::addEdge(tail, head);
  }
} /* namespace gum */

==> agrum-2.2.1-1-x86_64.pkg.tar.zst: usr/include/agrum/base/graphs/PDAG_inl.h <==
/****************************************************************************
 *   This file is part of the aGrUM/pyAgrum library.                        *
 *                                                                          *
 *   Copyright (c) 2005-2025 by                                             *
 *       - Pierre-Henri WUILLEMIN(_at_LIP6)                                 *
 *       - Christophe GONZALES(_at_AMU)                                     *
 *                                                                          *
 *   The aGrUM/pyAgrum library is free software; you can redistribute it    *
 *   and/or modify it under the terms of either :                           *
 *                                                                          *
 *    - the GNU Lesser General Public License as published by               *
 *      the Free Software Foundation, either version 3 of the License,      *
 *      or (at your option) any later version,                              *
 *    - the MIT license (MIT),                                              *
 *    - or both in dual license, as here.                                   *
 *                                                                          *
 *   (see https://agrum.gitlab.io/articles/dual-licenses-lgplv3mit.html)    *
 *                                                                          *
 *   This aGrUM/pyAgrum library is distributed in the hope that it will be  *
 *   useful, but WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,          *
 *   INCLUDING BUT NOT LIMITED TO THE WARRANTIES MERCHANTABILITY or FITNESS *
 *   FOR A PARTICULAR PURPOSE  AND NONINFRINGEMENT. IN NO EVENT SHALL THE   *
 *   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER *
 *   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,        *
 *   ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR  *
 *   OTHER DEALINGS IN THE SOFTWARE.                                        *
 *                                                                          *
 *   See LICENCES for more details.                                         *
 *                                                                          *
 *   SPDX-FileCopyrightText: Copyright 2005-2025                            *
 *       - Pierre-Henri WUILLEMIN(_at_LIP6)                                 *
 *       - Christophe GONZALES(_at_AMU)                                     *
 *   SPDX-License-Identifier: LGPL-3.0-or-later OR MIT                      *
 *                                                                          *
 *   Contact  : info_at_agrum_dot_org                                       *
 *   homepage : http://agrum.gitlab.io                                      *
 *   gitlab   : https://gitlab.com/agrumery/agrum                           *
 *                                                                          *
 ****************************************************************************/


/** @file
 * @brief Inline implementation of Base classes for directed acylic graphs
 *
 * @author Pierre-Henri WUILLEMIN(_at_LIP6) & Christophe GONZALES(_at_AMU)
 *
 */

// to ease IDE parser
#include <agrum/base/core/exceptions.h>
#include <agrum/base/graphs/PDAG.h>

namespace gum {

  INLINE PDAG& PDAG::operator=(const PDAG& g) {
    // avoid self assignment
    if (this != &g) { MixedGraph::operator=(g); }

    return *this;
  }

  INLINE void PDAG::addArc(const NodeId tail, const NodeId head) {
    if (head == tail) {
      GUM_ERROR(InvalidDirectedCycle, "Add a mono-cycle in a PDAG for node " << head)
    }
    if (this->hasDirectedPath(head, tail)) {
      GUM_ERROR(InvalidDirectedCycle,
                "Add a directed cycle in a PDAG : between " << head << " and " << tail)
    }
    if (this->hasMixedReallyOrientedPath(head, tail)) {
      GUM_ERROR(InvalidPartiallyDirectedCycle,
                "Add a partial directed cycle in a PDAG : between " << head << " and " << tail)
    }

    // checking whether tail and head do belong to the graph is performed
    // within class DiGraph
    MixedGraph::addArc(tail, head);
  }

  INLINE void PDAG::addEdge(const NodeId tail, const NodeId head) {
    if (head == tail) { GUM_ERROR(InvalidDirectedCycle, "Add a mono-cycle in a PDAG !") }

    if (this->hasMixedReallyOrientedPath(head, tail)
        || this->hasMixedReallyOrientedPath(tail, head)) {
      GUM_ERROR(InvalidPartiallyDirectedCycle,
                "Add a partial directed cycle in a PDAG : between " << head << " and " << tail)
    }

    // checking whether tail and head do belong to the graph is performed
    // within class DiGraph
    MixedGraph::addEdge(tail, head);
  }
} /* namespace gum */
